#include "StdAfx.h"
#include "AESCrypto.h"
#include <WinCrypt.h>

#include "openssl/aes.h"
#include "openssl/evp.h"
#include "Cryptopp561/sha.h"


typedef struct _pwdcrypt_t
{
	EVP_CIPHER_CTX ctx;
	char key[16];
	char iv[16];
} pwdcrypt_t;

CAESCrypto::CAESCrypto(void)
{
}

CAESCrypto::~CAESCrypto(void)
{
}


// key_base_table  seed  ̿ؼ crypt_key  .
void * CAESCrypto::pwdcrypt_init(const char * seed, const char * key_base_table, int key_base_table_len)
{
	int i_key;
	int i_seed;
	int seed_len = strlen(seed);
	pwdcrypt_t * phandle = (pwdcrypt_t *)malloc(sizeof(pwdcrypt_t));

	if (NULL == phandle)
	{
		return NULL;
	}

	memset(phandle, 0, sizeof(pwdcrypt_t));
	EVP_CIPHER_CTX_init(&phandle->ctx);

	for(i_key = 0, i_seed = 0 ;
		i_key < (int)sizeof(phandle->key) ;
		i_key++, (i_seed = (i_seed + 1) % seed_len))
	{
		int index_of_table = seed[i_seed] % key_base_table_len;
		phandle->key[i_key] = key_base_table[ index_of_table ];
	}

	return (void *)phandle;
}

void CAESCrypto::pwdcrypt_free(void * handle)
{
	pwdcrypt_t * p = (pwdcrypt_t *)handle;
	EVP_CIPHER_CTX_cleanup(&p->ctx);

	free((pwdcrypt_t *)handle);
}

bool CAESCrypto::pwdcrypt_encrypt(const char * data, int data_len,
					  unsigned char * pOutBuffer, int * pOutBufferLen,
					  const char * seed,
					  const char * key_base_table, int key_base_table_len)
{
	if(data_len == 0)
	{
		*pOutBufferLen = 0;
		return true;
	}

	pwdcrypt_t * phandle = (pwdcrypt_t *)pwdcrypt_init(seed, key_base_table, key_base_table_len);

	if (NULL == phandle)
	{
		return false;
	}

	EVP_EncryptInit(&phandle->ctx, EVP_aes_128_cbc(), (const unsigned char *)phandle->key, (unsigned char *)phandle->iv);

	EVP_EncryptUpdate(&phandle->ctx, pOutBuffer, pOutBufferLen, (const unsigned char *)data, data_len);

	int remainLen = 0;
	EVP_EncryptFinal(&phandle->ctx, pOutBuffer + *pOutBufferLen, &remainLen);

	(*pOutBufferLen) += remainLen;

	pwdcrypt_free(phandle);
	return true;
}

bool CAESCrypto::pwdcrypt_decrypt(const unsigned char * data, int data_len,
					  char * pOutBuffer, int * pOutBufferLen,
					  const char * seed,
					  const char * key_base_table, int key_base_table_len)
{
	if(data_len == 0)
	{
		*pOutBufferLen = 0;
		return true;
	}
    
	pwdcrypt_t * phandle = (pwdcrypt_t *)pwdcrypt_init(seed, key_base_table, key_base_table_len);

	if (NULL == phandle)
	{
		return false;
	}
	
	EVP_DecryptInit(&phandle->ctx, EVP_aes_128_cbc(), (const unsigned char *)phandle->key, (unsigned char *)phandle->iv);

	EVP_DecryptUpdate(&phandle->ctx, (unsigned char *)pOutBuffer, pOutBufferLen, data, data_len);
	
	int remainLen = 0;
	EVP_DecryptFinal(&phandle->ctx, (unsigned char *)pOutBuffer + *pOutBufferLen, &remainLen);

	*pOutBufferLen += remainLen;
	pOutBuffer[*pOutBufferLen] = '\0';

	pwdcrypt_free(phandle);
	return true;
}

BOOL CAESCrypto::Base64Encode(const BYTE *pbSrcData, int nSrcLen, LPSTR szDest, int *pnDestLen, DWORD dwFlags) throw()
{
	static const char s_chBase64EncodingTable[64] = {
		'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q',
		'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 'g',	'h',
		'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y',
		'z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '+', '/' };

		if (!pbSrcData || !szDest || !pnDestLen)
		{
			return FALSE;
		}

		int nWritten( 0 );
		int nLen1( (nSrcLen/3)*4 );
		int nLen2( nLen1/76 );
		int nLen3( 19 );

		for (int i=0; i<=nLen2; i++)
		{
			if (i==nLen2)
				nLen3 = (nLen1%76)/4;

			for (int j=0; j<nLen3; j++)
			{
				DWORD dwCurr(0);
				for (int n=0; n<3; n++)
				{
					dwCurr |= *pbSrcData++;
					dwCurr <<= 8;
				}
				for (int k=0; k<4; k++)
				{
					BYTE b = (BYTE)(dwCurr>>26);
					*szDest++ = s_chBase64EncodingTable[b];
					dwCurr <<= 6;
				}
			}
			nWritten+= nLen3*4;

			if ((dwFlags & BASE64_FLAG_NOCRLF)==0)
			{
				*szDest++ = '\r';
				*szDest++ = '\n';
				nWritten+= 2;
			}
		}

		if (nWritten && (dwFlags & BASE64_FLAG_NOCRLF)==0)
		{
			szDest-= 2;
			nWritten -= 2;
		}

		nLen2 = nSrcLen%3 ? nSrcLen%3 + 1 : 0;
		if (nLen2)
		{
			DWORD dwCurr(0);
			for (int n=0; n<3; n++)
			{
				if (n<(nSrcLen%3))
					dwCurr |= *pbSrcData++;
				dwCurr <<= 8;
			}
			for (int k=0; k<nLen2; k++)
			{
				BYTE b = (BYTE)(dwCurr>>26);
				*szDest++ = s_chBase64EncodingTable[b];
				dwCurr <<= 6;
			}
			nWritten+= nLen2;
			if ((dwFlags & BASE64_FLAG_NOPAD)==0)
			{
				nLen3 = nLen2 ? 4-nLen2 : 0;
				for (int j=0; j<nLen3; j++)
				{
					*szDest++ = '=';
				}
				nWritten+= nLen3;
			}
		}

		*pnDestLen = nWritten;
		return TRUE;
}

BOOL CAESCrypto::GetSHA256Hash(char *buffer,           //input buffer
							   DWORD dwBufferSize,       //input buffer size
							   BYTE *byteFinalHash,      //ouput hash buffer
							   DWORD *dwFinalHashSize    //input/output final buffer size
							   )
{
	//DWORD dwStatus = 0;
	BOOL bResult = FALSE;
	HCRYPTPROV hProv = 0;
	HCRYPTHASH hHash = 0;
	//BYTE *byteHash;
	DWORD cbHashSize = 0;
	CString strErrorMsg;

	// Get handle to the crypto provider
	if (!CryptAcquireContext(&hProv, NULL, NULL, PROV_RSA_AES, CRYPT_VERIFYCONTEXT))
	{
		strErrorMsg.Format(_T("CryptAcquireContext failed, Error=0x%.8x"), GetLastError());
		GET_APP()->m_pMessageBoxManager->CreateMessageBox(NULL, strErrorMsg);
		return bResult;
	}

	//Specify the Hash Algorithm here
	if (!CryptCreateHash(hProv, CALG_SHA_256, 0, 0, &hHash))
	{
		strErrorMsg.Format(_T("CryptCreateHash failed,  Error=0x%.8x"), GetLastError());
		GET_APP()->m_pMessageBoxManager->CreateMessageBox(NULL, strErrorMsg);
		goto EndHash;
	}

	//Create the hash with input buffer
	if (!CryptHashData(hHash, (const BYTE*) buffer, dwBufferSize, 0))
	{
		strErrorMsg.Format(_T("CryptHashData failed,  Error=0x%.8x"), GetLastError());
		GET_APP()->m_pMessageBoxManager->CreateMessageBox(NULL, strErrorMsg);
		goto EndHash;
	}

	//Get the final hash size 
	DWORD dwCount = sizeof(DWORD);
	if(!CryptGetHashParam(hHash, HP_HASHSIZE, (BYTE *)&cbHashSize, &dwCount, 0)) 
	{
		strErrorMsg.Format(_T("CryptGetHashParam failed, Error=0x%.8x"), GetLastError());
		GET_APP()->m_pMessageBoxManager->CreateMessageBox(NULL, strErrorMsg);
		goto EndHash;
	}

	//check if the output buffer is enough to copy the hash data
	if( *dwFinalHashSize < cbHashSize )
	{
		strErrorMsg.Format(_T("Output buffer (%d) is not sufficient, Required Size = %d"), *dwFinalHashSize, cbHashSize);
		GET_APP()->m_pMessageBoxManager->CreateMessageBox(NULL, strErrorMsg);
		goto EndHash;
	}

	//Now get the computed hash 
	if (CryptGetHashParam(hHash, HP_HASHVAL, byteFinalHash, dwFinalHashSize, 0))
	{
		Trace(_T("********** Hash Computed successfully "));
		bResult = TRUE; 
	}
	else
	{
		strErrorMsg.Format(_T("CryptGetHashParam failed,  Error=0x%.8x"), GetLastError());
		GET_APP()->m_pMessageBoxManager->CreateMessageBox(NULL, strErrorMsg);
	}

EndHash:

	if( hHash )
		CryptDestroyHash(hHash);

	if( hProv )
		CryptReleaseContext(hProv, 0);

	return bResult; 
}

BOOL CAESCrypto::GetSHA256(char* pSrc, size_t dwSrcLength, BYTE* byteSHA256Hash)
{
	unsigned int			nSize		= 0;
	PBYTE					pBuf		= NULL;
	size_t					nHexSize	= 0;
	PBYTE					pHex		= NULL;
	CryptoPP::SHA256		cSHA256;

	if (NULL == pSrc)
	{
		return FALSE;
	}

	// hash .
	cSHA256.CalculateDigest(byteSHA256Hash, (LPBYTE)pSrc, dwSrcLength);

	return TRUE;
}
